@@ -1,31 +1,18 @@
-extern crate ansi_term;
-extern crate cargo;
-extern crate chrono;
-#[macro_use]
-extern crate clap;
-#[macro_use]
-extern crate debcargo;
-extern crate flate2;
-extern crate glob;
-extern crate itertools;
-extern crate semver;
-extern crate semver_parser;
-extern crate tar;
-extern crate tempfile;
-extern crate walkdir;
-
-use ansi_term::Colour::Red;
-use clap::{App, AppSettings, ArgMatches, SubCommand};
 use std::env;
 use std::fs;
 use std::io::{BufRead, BufReader};
 use std::path::{Path, PathBuf};
 
+use ansi_term::Colour::Red;
+use anyhow::Context;
+use clap::{crate_authors, crate_version, App, AppSettings, ArgMatches, SubCommand};
+
 use debcargo::config::{parse_config, Config};
 use debcargo::crates::{update_crates_io, CrateInfo};
-use debcargo::debian::{self, BaseInfo};
-use debcargo::errors::*;
+use debcargo::debian::{self, DebInfo};
+use debcargo::errors::Result;
 use debcargo::util;
+use debcargo::{debcargo_info, debcargo_warn};
 
 fn lookup_fixmes(srcdir: &Path) -> Result<Vec<PathBuf>> {
     let mut fixme_files = Vec::new();
@@ -70,24 +57,29 @@ fn do_package(matches: &ArgMatches) -> Result<()> {
     let overlay_write_back = !matches.is_present("no-overlay-write-back");
     let copyright_guess_harder = matches.is_present("copyright-guess-harder");
 
-    let mut crate_info = CrateInfo::new(crate_name, version)?;
-    let pkgbase = BaseInfo::new(
+    let crate_path = config.crate_src_path(config_path);
+
+    let mut crate_info = match crate_path {
+        Some(p) => CrateInfo::new_with_local_crate(crate_name, version, &p)?,
+        None => CrateInfo::new(crate_name, version)?,
+    };
+    crate_info.set_includes_excludes(config.orig_tar_excludes(), config.orig_tar_whitelist());
+    let deb_info = DebInfo::new(
         crate_name,
         &crate_info,
         crate_version!(),
         config.semver_suffix,
     );
+    let pkg_srcdir = Path::new(directory.unwrap_or_else(|| deb_info.package_source_dir()));
+    let source_modified = crate_info.extract_crate(pkg_srcdir)?;
 
-    let pkg_srcdir = Path::new(directory.unwrap_or(pkgbase.package_source_dir()));
     let orig_tar_gz = pkg_srcdir
         .parent()
         .unwrap()
-        .join(pkgbase.orig_tarball_path());
-    crate_info.set_includes_excludes(config.orig_tar_excludes(), config.orig_tar_whitelist());
-    let source_modified = crate_info.extract_crate(pkg_srcdir)?;
+        .join(deb_info.orig_tarball_path());
     debian::prepare_orig_tarball(&crate_info, &orig_tar_gz, source_modified, pkg_srcdir)?;
     debian::prepare_debian_folder(
-        &pkgbase,
+        &deb_info,
         &mut crate_info,
         pkg_srcdir,
         config_path,
@@ -119,17 +111,15 @@ fn do_package(matches: &ArgMatches) -> Result<()> {
             match config_path {
                 None => debcargo_warn!("\t •  Write a config file and use it with --config"),
                 Some(c) => {
-                    debcargo_warn!(format!(
-                        "\t •  Add or edit overrides in your config file:"
-                    ));
-                    debcargo_warn!(format!("\t    {}", rel_p(&c, &curdir)));
+                    debcargo_warn!("\t •  Add or edit overrides in your config file:");
+                    debcargo_warn!("\t    {}", rel_p(&c, &curdir));
                 }
             };
-            match config.overlay {
-                None => debcargo_warn!(format!("\t •  Create an overlay directory and add it to your config file with overlay = \"/path/to/overlay\"")),
-                Some(_) => {
-                    debcargo_warn!(format!("\t •  Add or edit files in your overlay directory:"));
-                    debcargo_warn!(format!("\t    {}", rel_p(&config.overlay_dir(config_path).unwrap(), &curdir)));
+            match config.overlay_dir(config_path) {
+                None => debcargo_warn!("\t •  Create an overlay directory and add it to your config file with overlay = \"/path/to/overlay\""),
+                Some(p) => {
+                    debcargo_warn!("\t •  Add or edit files in your overlay directory:");
+                    debcargo_warn!("\t    {}", rel_p(&p, &curdir));
                 }
             }
         }
@@ -143,9 +133,9 @@ fn do_deb_src_name(matches: &ArgMatches) -> Result<()> {
     let version = matches.value_of("version");
 
     let crate_info = CrateInfo::new_with_update(crate_name, version, false)?;
-    let pkgbase = BaseInfo::new(crate_name, &crate_info, crate_version!(), version.is_some());
+    let deb_info = DebInfo::new(crate_name, &crate_info, crate_version!(), version.is_some());
 
-    println!("{}", pkgbase.package_name());
+    println!("{}", deb_info.package_name());
     Ok(())
 }
 
@@ -155,8 +145,8 @@ fn do_extract(matches: &ArgMatches) -> Result<()> {
     let directory = matches.value_of("directory");
 
     let crate_info = CrateInfo::new(crate_name, version)?;
-    let pkgbase = BaseInfo::new(crate_name, &crate_info, crate_version!(), false);
-    let pkg_srcdir = Path::new(directory.unwrap_or(pkgbase.package_source_dir()));
+    let deb_info = DebInfo::new(crate_name, &crate_info, crate_version!(), false);
+    let pkg_srcdir = Path::new(directory.unwrap_or_else(|| deb_info.package_source_dir()));
 
     crate_info.extract_crate(pkg_srcdir)?;
     Ok(())
