@@ -1,11 +1,16 @@
+use serde::Deserialize;
 use toml;
 
-use errors::*;
+use crate::errors::*;
+use crate::util::vec_opt_iter;
+
 use std::collections::HashMap;
 use std::fs::File;
 use std::io::Read;
 use std::path::{Path, PathBuf};
-use util::vec_opt_iter;
+
+pub const RUST_MAINT: &str =
+    "Debian Rust Maintainers <pkg-rust-maintainers@alioth-lists.debian.net>";
 
 #[derive(Deserialize, Debug, Clone)]
 #[serde(default)]
@@ -17,9 +22,12 @@ pub struct Config {
     pub excludes: Option<Vec<String>>,
     pub whitelist: Option<Vec<String>>,
     pub allow_prerelease_deps: bool,
+    pub crate_src_path: Option<PathBuf>,
     pub summary: String,
     pub description: String,
+    pub maintainer: String,
     pub uploaders: Option<Vec<String>>,
+    pub collapse_features: bool,
 
     pub source: Option<SourceOverride>,
     pub packages: Option<HashMap<String, PackageOverride>>,
@@ -47,6 +55,7 @@ pub struct PackageOverride {
     provides: Option<Vec<String>>,
     extra_lines: Option<Vec<String>>,
     test_is_broken: Option<bool>,
+    test_depends: Option<Vec<String>>,
 }
 
 impl Default for Config {
@@ -59,9 +68,12 @@ impl Default for Config {
             excludes: None,
             whitelist: None,
             allow_prerelease_deps: false,
+            crate_src_path: None,
             summary: "".to_string(),
             description: "".to_string(),
+            maintainer: RUST_MAINT.to_string(),
             uploaders: None,
+            collapse_features: false,
             source: None,
             packages: None,
         }
@@ -77,17 +89,11 @@ impl Config {
     }
 
     pub fn overlay_dir(&self, config_path: Option<&Path>) -> Option<PathBuf> {
-        self.overlay
-            .as_ref()
-            .map(|p| config_path.unwrap().parent().unwrap().join(p))
+        Some(config_path?.parent()?.join(self.overlay.as_ref()?))
     }
 
-    pub fn is_source_present(&self) -> bool {
-        self.source.is_some()
-    }
-
-    pub fn is_packages_present(&self) -> bool {
-        self.packages.is_some()
+    pub fn crate_src_path(&self, config_path: Option<&Path>) -> Option<PathBuf> {
+        Some(config_path?.parent()?.join(self.crate_src_path.as_ref()?))
     }
 
     pub fn orig_tar_excludes(&self) -> Option<&Vec<String>> {
@@ -98,128 +104,95 @@ impl Config {
         self.whitelist.as_ref()
     }
 
-    pub fn policy_version(&self) -> Option<&str> {
-        if let Some(ref s) = self.source {
-            if let Some(ref policy) = s.policy {
-                return Some(policy);
-            }
-        }
-        None
-    }
-
-    pub fn homepage(&self) -> Option<&str> {
-        if let Some(ref s) = self.source {
-            if let Some(ref homepage) = s.homepage {
-                return Some(homepage);
-            }
-        }
-        None
-    }
-
-    pub fn build_depends(&self) -> Option<&Vec<String>> {
-        self.source.as_ref().and_then(|s| s.build_depends.as_ref())
+    pub fn maintainer(&self) -> &str {
+        self.maintainer.as_str()
     }
 
     pub fn uploaders(&self) -> Option<&Vec<String>> {
         self.uploaders.as_ref()
     }
 
-    pub fn build_depends_excludes(&self) -> Option<&Vec<String>> {
-        self.source
-            .as_ref()
-            .and_then(|s| s.build_depends_excludes.as_ref())
-    }
+    // Source shortcuts
 
     pub fn section(&self) -> Option<&str> {
-        if let Some(ref s) = self.source {
-            if let Some(ref section) = s.section {
-                return Some(section);
-            }
-        }
-        None
+        Some(self.source.as_ref()?.section.as_ref()?)
     }
 
-    pub fn package_section(&self, key: PackageKey) -> Option<&str> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.section.as_ref().map(|s| s.as_str()))
-        })
+    pub fn policy_version(&self) -> Option<&str> {
+        Some(self.source.as_ref()?.policy.as_ref()?)
     }
 
-    pub fn package_summary(&self, key: PackageKey) -> Option<(&str, &str)> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key)).map(|package| {
-                let s = match package.summary {
-                    Some(ref s) => s,
-                    None => "",
-                };
-                let d = match package.description {
-                    Some(ref d) => d,
-                    None => "",
-                };
-                (s, d)
-            })
-        })
-    }
-
-    pub fn package_depends(&self, key: PackageKey) -> Option<&Vec<String>> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.depends.as_ref())
-        })
-    }
-
-    pub fn package_recommends(&self, key: PackageKey) -> Option<&Vec<String>> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.recommends.as_ref())
-        })
-    }
-
-    pub fn package_suggests(&self, key: PackageKey) -> Option<&Vec<String>> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.suggests.as_ref())
-        })
-    }
-
-    pub fn package_provides(&self, key: PackageKey) -> Option<&Vec<String>> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.provides.as_ref())
-        })
-    }
-
-    pub fn package_extra_lines(&self, key: PackageKey) -> Option<&Vec<String>> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.extra_lines.as_ref())
-        })
-    }
-
-    pub fn package_test_is_broken(&self, key: PackageKey) -> Option<bool> {
-        self.packages.as_ref().and_then(|pkg| {
-            pkg.get(&package_key_string(key))
-                .and_then(|package| package.test_is_broken)
-        })
+    pub fn homepage(&self) -> Option<&str> {
+        Some(self.source.as_ref()?.homepage.as_ref()?)
     }
 
     pub fn vcs_git(&self) -> Option<&str> {
-        if let Some(ref s) = self.source {
-            if let Some(ref vcs_git) = s.vcs_git {
-                return Some(vcs_git);
-            }
-        }
-        None
+        Some(self.source.as_ref()?.vcs_git.as_ref()?)
     }
 
     pub fn vcs_browser(&self) -> Option<&str> {
-        if let Some(ref s) = self.source {
-            if let Some(ref vcs_browser) = s.vcs_browser {
-                return Some(vcs_browser);
-            }
-        }
-        None
+        Some(self.source.as_ref()?.vcs_browser.as_ref()?)
+    }
+
+    pub fn build_depends(&self) -> Option<&Vec<String>> {
+        self.source.as_ref()?.build_depends.as_ref()
+    }
+
+    pub fn build_depends_excludes(&self) -> Option<&Vec<String>> {
+        self.source.as_ref()?.build_depends_excludes.as_ref()
+    }
+
+    // Packages shortcuts
+
+    fn with_package<'a, T, F: FnOnce(&'a PackageOverride) -> Option<T>>(
+        &'a self,
+        key: PackageKey,
+        f: F,
+    ) -> Option<T> {
+        self.packages
+            .as_ref()?
+            .get(&package_key_string(key))
+            .and_then(f)
+    }
+
+    pub fn package_section(&self, key: PackageKey) -> Option<&str> {
+        self.with_package(key, |pkg| pkg.section.as_ref().map(|s| s.as_str()))
+    }
+
+    pub fn package_summary(&self, key: PackageKey) -> Option<&str> {
+        self.with_package(key, |pkg| pkg.summary.as_ref().map(|s| s.as_str()))
+    }
+
+    pub fn package_description(&self, key: PackageKey) -> Option<&str> {
+        self.with_package(key, |pkg| pkg.description.as_ref().map(|s| s.as_str()))
+    }
+
+    pub fn package_depends(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.depends.as_ref())
+    }
+
+    pub fn package_recommends(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.recommends.as_ref())
+    }
+
+    pub fn package_suggests(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.suggests.as_ref())
+    }
+
+    pub fn package_provides(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.provides.as_ref())
+    }
+
+    pub fn package_extra_lines(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.extra_lines.as_ref())
+    }
+
+    pub fn package_test_is_broken(&self, key: PackageKey) -> Option<bool> {
+        self.with_package(key, |pkg| pkg.test_is_broken)
+    }
+
+    pub fn package_test_depends(&self, key: PackageKey) -> Option<&Vec<String>> {
+        self.with_package(key, |pkg| pkg.test_depends.as_ref())
     }
 }
 
@@ -232,13 +205,13 @@ pub fn parse_config(src: &Path) -> Result<Config> {
 }
 
 pub fn package_field_for_feature<'a>(
-    get_field: &'a Fn(PackageKey) -> Option<&'a Vec<String>>,
+    get_field: &'a dyn Fn(PackageKey) -> Option<&'a Vec<String>>,
     feature: PackageKey,
     f_provides: &[&str],
 ) -> Vec<String> {
     Some(feature)
         .into_iter()
-        .chain(f_provides.into_iter().map(|s| PackageKey::feature(s)))
+        .chain(f_provides.iter().map(|s| PackageKey::feature(s)))
         .map(move |f| vec_opt_iter(get_field(f)))
         .flatten()
         .map(|s| s.to_string())
