@@ -1,15 +1,18 @@
+use anyhow::{format_err, Error};
 use cargo::{
     core::manifest::ManifestMetadata,
     core::registry::PackageRegistry,
+    core::source::MaybePackage,
     core::{
         Dependency, EitherManifest, FeatureValue, Manifest, Package, PackageId, Registry, Source,
-        SourceId, Summary, Target, TargetKind,
+        SourceId, Summary, Target, TargetKind, Workspace,
     },
+    ops,
+    ops::PackageOpts,
     sources::registry::RegistrySource,
-    util::{toml::read_manifest, FileLock},
+    util::{interning::InternedString, toml::read_manifest, FileLock},
     Config,
 };
-use failure::Error;
 use filetime::{set_file_times, FileTime};
 use flate2::read::GzDecoder;
 use glob::Pattern;
@@ -25,8 +28,8 @@ use std::hash::{Hash, Hasher};
 use std::io::{self, Read, Write};
 use std::path::{Path, PathBuf};
 
-use errors::*;
-use util::vec_opt_iter;
+use crate::errors::*;
+use crate::util::vec_opt_iter;
 
 pub struct CrateInfo {
     package: Package,
@@ -45,17 +48,6 @@ fn hash<H: Hash>(hashable: &H) -> u64 {
     hasher.finish()
 }
 
-fn traverse_depth<'a>(map: &BTreeMap<&'a str, Vec<&'a str>>, key: &'a str) -> Vec<&'a str> {
-    let mut x = Vec::new();
-    if let Some(pp) = (*map).get(key) {
-        x.extend(pp);
-        for p in pp {
-            x.extend(traverse_depth(map, p));
-        }
-    }
-    x
-}
-
 fn fetch_candidates(registry: &mut PackageRegistry, dep: &Dependency) -> Result<Vec<Summary>> {
     let mut summaries = registry.query_vec(dep, false)?;
     summaries.sort_by(|a, b| b.package_id().partial_cmp(&a.package_id()).unwrap());
@@ -76,6 +68,89 @@ impl CrateInfo {
         CrateInfo::new_with_update(crate_name, version, true)
     }
 
+    pub fn new_with_local_crate(
+        crate_name: &str,
+        version: Option<&str>,
+        crate_path: &Path,
+    ) -> Result<CrateInfo> {
+        let config = Config::default()?;
+        let crate_path = crate_path.canonicalize()?;
+        let source_id = SourceId::for_path(&crate_path)?;
+
+        let (package, crate_file) = {
+            let yanked_whitelist = HashSet::new();
+
+            let mut source = source_id.load(&config, &yanked_whitelist)?;
+            source.update()?;
+
+            let package_id = match version {
+                None | Some("") => {
+                    let dep = Dependency::parse_no_deprecated(crate_name, None, source_id)?;
+                    let mut package_id: Option<PackageId> = None;
+                    source.query(&dep, &mut |p| package_id = Some(p.package_id()))?;
+                    package_id.unwrap()
+                }
+                Some(version) => PackageId::new(crate_name, version, source_id)?,
+            };
+
+            let maybe_package = source.download(package_id)?;
+            let package = match maybe_package {
+                MaybePackage::Ready(p) => Ok(p),
+                _ => Err(format_err!(
+                    "Failed to 'download' local crate {} from {}",
+                    crate_name,
+                    crate_path.display()
+                )),
+            }?;
+
+            let crate_file = {
+                let workspace = Workspace::ephemeral(package.clone(), &config, None, true)?;
+
+                let opts = PackageOpts {
+                    config: &config,
+                    verify: false,
+                    list: false,
+                    check_metadata: true,
+                    allow_dirty: true,
+                    all_features: true,
+                    no_default_features: false,
+                    jobs: None,
+                    targets: Vec::new(),
+                    features: Vec::new(),
+                };
+
+                // as of cargo 0.41 this returns a FileLock with a temp path, instead of the one
+                // it got renamed to
+                if ops::package(&workspace, &opts)?.is_none() {
+                    return Err(format_err!(
+                        "Failed to assemble crate file for local crate {} at {}\n",
+                        crate_name,
+                        crate_path.display()
+                    ));
+                }
+                let filename = format!("{}-{}.crate", crate_name, package_id.version().to_string());
+                workspace
+                    .target_dir()
+                    .join("package")
+                    .open_rw(&filename, &config, "crate file")?
+            };
+
+            (package.clone(), crate_file)
+        };
+
+        let manifest = package.manifest().clone();
+
+        Ok(CrateInfo {
+            package,
+            manifest,
+            crate_file,
+            config,
+            source_id,
+            excludes: vec![],
+            includes: vec![],
+        })
+    }
+
     pub fn new_with_update(
         crate_name: &str,
         version: Option<&str>,
@@ -122,7 +197,7 @@ impl CrateInfo {
             drop(lock);
             let pkgids = summaries
                 .into_iter()
-                .map(|s| s.package_id().clone())
+                .map(|s| s.package_id())
                 .collect::<Vec<_>>();
             let pkgid = pkgids.iter().max().ok_or_else(|| {
                 format_err!(
@@ -146,22 +221,31 @@ impl CrateInfo {
         };
 
         Ok(CrateInfo {
-            package: package,
-            manifest: manifest,
-            crate_file: crate_file,
-            config: config,
-            source_id: source_id,
+            package,
+            manifest,
+            crate_file,
+            config,
+            source_id,
             excludes: vec![],
             includes: vec![],
         })
     }
 
-    pub fn targets(&self) -> &[Target] {
-        self.manifest.targets()
+    pub fn package(&self) -> &Package {
+        &self.package
     }
 
     pub fn version(&self) -> &Version {
-        self.manifest.summary().package_id().version()
+        self.package_id().version()
+    }
+
+    pub fn semver(&self) -> String {
+        match *self.package_id().version() {
+            Version {
+                major: 0, minor, ..
+            } => format!("0.{}", minor),
+            Version { major, .. } => format!("{}", major),
+        }
     }
 
     pub fn manifest(&self) -> &Manifest {
@@ -175,24 +259,52 @@ impl CrateInfo {
         Ok(self)
     }
 
-    pub fn checksum(&self) -> Option<&str> {
-        self.manifest.summary().checksum()
-    }
-
-    pub fn package_id(&self) -> PackageId {
-        self.manifest.summary().package_id()
-    }
-
     pub fn metadata(&self) -> &ManifestMetadata {
         self.manifest.metadata()
     }
 
+    pub fn targets(&self) -> &[Target] {
+        self.manifest.targets()
+    }
+
+    pub fn is_lib(&self) -> bool {
+        let mut lib = false;
+        for target in self.manifest.targets() {
+            match *target.kind() {
+                TargetKind::Lib(_) => {
+                    lib = true;
+                    break;
+                }
+                _ => continue,
+            }
+        }
+        lib
+    }
+
+    pub fn get_binary_targets(&self) -> Vec<&str> {
+        let mut bins = Vec::new();
+        for target in self.manifest.targets() {
+            match *target.kind() {
+                TargetKind::Bin => {
+                    bins.push(target.name());
+                }
+                _ => continue,
+            }
+        }
+        bins.sort();
+        bins
+    }
+
     pub fn summary(&self) -> &Summary {
         self.manifest.summary()
     }
 
-    pub fn package(&self) -> &Package {
-        &self.package
+    pub fn checksum(&self) -> Option<&str> {
+        self.manifest.summary().checksum()
+    }
+
+    pub fn package_id(&self) -> PackageId {
+        self.manifest.summary().package_id()
     }
 
     pub fn crate_file(&self) -> &FileLock {
@@ -204,16 +316,18 @@ impl CrateInfo {
     }
 
     pub fn dev_dependencies(&self) -> Vec<Dependency> {
-        use cargo::core::dependency::Kind;
+        use cargo::core::dependency::DepKind;
         let mut deps = vec![];
         for dep in self.dependencies() {
-            if dep.kind() == Kind::Development {
+            if dep.kind() == DepKind::Development {
                 deps.push(dep.clone())
             }
         }
         deps
     }
 
+    /// Collect information about the dependency structure of features and
+    /// their external crate dependencies, in a simple output format.
     pub fn all_dependencies_and_features(
         &self,
     ) -> BTreeMap<
@@ -225,12 +339,12 @@ impl CrateInfo {
         ),
     > // dependencies: other packages
     {
-        use cargo::core::dependency::Kind;
+        use cargo::core::dependency::DepKind;
 
         let mut deps_by_name: BTreeMap<&str, Vec<&Dependency>> = BTreeMap::new();
         for dep in self.dependencies() {
             // we treat build-dependencies also as dependencies in Debian
-            if dep.kind() != Kind::Development {
+            if dep.kind() != DepKind::Development {
                 let s = dep.name_in_toml().as_str();
                 deps_by_name.entry(s).or_default().push(dep);
             }
@@ -241,8 +355,7 @@ impl CrateInfo {
 
         // calculate dependencies of this crate's features
         for (feature, deps) in self.manifest.summary().features() {
-            let mut feature_deps = vec![""];
-            // always need "", because in dh-cargo we symlink /usr/share/doc/{$feature => $main} pkg
+            let mut feature_deps: Vec<&str> = vec![];
             let mut other_deps: Vec<Dependency> = Vec::new();
             for dep in deps {
                 use self::FeatureValue::*;
@@ -261,13 +374,17 @@ impl CrateInfo {
                         // unwrap is ok, valid Cargo.toml files must have this
                         for &dep in deps_by_name.get(dep_name.as_str()).unwrap() {
                             let mut dep = dep.clone();
-                            dep.set_features(vec![dep_feature.to_string()]);
+                            dep.set_features(vec![InternedString::new(dep_feature)]);
                             dep.set_default_features(false);
                             other_deps.push(dep);
                         }
                     }
                 }
             }
+            if feature_deps.is_empty() {
+                // everything depends on bare library
+                feature_deps.push("");
+            }
             features_with_deps.insert(feature.as_str(), (feature_deps, other_deps));
         }
 
@@ -295,154 +412,6 @@ impl CrateInfo {
         features_with_deps
     }
 
-    pub fn feature_all_deps<'a>(
-        &self,
-        features_with_deps: &'a BTreeMap<&str, (Vec<&str>, Vec<Dependency>)>,
-        feature: &str,
-    ) -> (Vec<&'a str>, Vec<Dependency>) {
-        let mut all_features = Vec::new();
-        let mut all_deps = Vec::new();
-        let &(ref ff, ref dd) = features_with_deps.get(feature).unwrap();
-        all_features.extend(ff.clone());
-        all_deps.extend(dd.clone());
-        for f in ff {
-            let (ff1, dd1) = self.feature_all_deps(&features_with_deps, f);
-            all_features.extend(ff1);
-            all_deps.extend(dd1);
-        }
-        (all_features, all_deps)
-    }
-
-    // Calculate Provides: in an attempt to reduce the number of binaries.
-    //
-    // Note: this mutates features_with_deps so you MUST run e.g.
-    // feature_all_deps *before* calling this.
-    //
-    // The algorithm is very simple and incomplete. e.g. it does not, yet
-    // simplify things like:
-    //   f1 depends on f2, f3
-    //   f2 depends on f4
-    //   f3 depends on f4
-    // into
-    //   f4 provides f1, f2, f3
-    pub fn calculate_provides<'a>(
-        &self,
-        features_with_deps: &mut BTreeMap<&'a str, (Vec<&'a str>, Vec<Dependency>)>,
-    ) -> BTreeMap<&'a str, Vec<&'a str>> {
-        // If any features have duplicate dependencies, deduplicate them by
-        // making all of the subsequent ones depend on the first one.
-        let mut features_rev_deps = BTreeMap::new();
-        for (&f, dep) in features_with_deps.iter() {
-            if !features_rev_deps.contains_key(dep) {
-                features_rev_deps.insert(dep.clone(), vec![]);
-            }
-            features_rev_deps.get_mut(dep).unwrap().push(f);
-        }
-        for (_, ff) in features_rev_deps.into_iter() {
-            let f0 = ff[0];
-            for f in &ff[1..] {
-                features_with_deps.insert(f, (vec!["", f0], vec![]));
-            }
-        }
-
-        // Calculate provides by following 0- or 1-length dependency lists.
-        let mut provides = BTreeMap::new();
-        let mut provided = Vec::new();
-        for (&f, (ref ff, ref dd)) in features_with_deps.iter() {
-            //debcargo_info!("provides considering: {:?}", &f);
-            if !dd.is_empty() {
-                continue;
-            }
-            assert!(!ff.is_empty() || f == "");
-            let k = if ff.len() == 1 {
-                // if A depends only on no-default-features (""), then
-                // no-default-features provides A.
-                assert!(ff[0] == "");
-                ff[0]
-            } else if ff.len() == 2 {
-                // if A depends on a single feature B, then B provides A.
-                assert!(ff[0] == "");
-                ff[1]
-            } else {
-                continue;
-            };
-            //debcargo_info!("provides still considering: {:?}", &f);
-            if !provides.contains_key(k) {
-                provides.insert(k, vec![]);
-            }
-            provides.get_mut(k).unwrap().push(f);
-            provided.push(f);
-        }
-
-        //debcargo_info!("provides-internal: {:?}", &provides);
-        //debcargo_info!("provided-internal: {:?}", &provided);
-        for p in provided {
-            features_with_deps.remove(p);
-        }
-
-        features_with_deps
-            .keys()
-            .map(|k| {
-                let mut pp = traverse_depth(&provides, k);
-                pp.sort();
-                (*k, pp)
-            })
-            .collect::<BTreeMap<_, _>>()
-    }
-
-    pub fn is_lib(&self) -> bool {
-        let mut lib = false;
-        for target in self.manifest.targets() {
-            match *target.kind() {
-                TargetKind::Lib(_) => {
-                    lib = true;
-                    break;
-                }
-                _ => continue,
-            }
-        }
-
-        lib
-    }
-
-    pub fn get_binary_targets(&self) -> Vec<&str> {
-        let mut bins = Vec::new();
-        for target in self.manifest.targets() {
-            match *target.kind() {
-                TargetKind::Bin => {
-                    bins.push(target.name());
-                }
-                _ => continue,
-            }
-        }
-        bins.sort();
-        bins
-    }
-
-    pub fn semver_suffix(&self) -> String {
-        let lib = self.is_lib();
-        let bins = self.get_binary_targets();
-
-        match *self.package_id().version() {
-            _ if !lib && !bins.is_empty() => "".to_string(),
-            Version {
-                major: 0, minor, ..
-            } => format!("-0.{}", minor),
-            Version { major, .. } => format!("-{}", major),
-        }
-    }
-
-    pub fn semver_uscan_pattern(&self) -> String {
-        // See `man uscan` description of @ANY_VERSION@ on how these
-        // regex patterns were built.
-        match *self.package_id().version() {
-            Version {
-                major: 0, minor, ..
-            } => format!("[-_]?(0\\.{}\\.\\d[\\-+\\.:\\~\\da-zA-Z]*)", minor),
-            Version { major, .. } => format!("[-_]?({}\\.\\d[\\-+\\.:\\~\\da-zA-Z]*)", major),
-        }
-    }
-
     pub fn get_summary_description(&self) -> (Option<String>, Option<String>) {
         let (summary, description) = if let Some(ref description) = self.metadata().description {
             // Convention these days seems to be to do manual text
@@ -516,13 +485,7 @@ impl CrateInfo {
             return Ok(true);
         }
         let suspicious = match path.extension() {
-            Some(ext) => {
-                if ext == "c" || ext == "a" {
-                    true
-                } else {
-                    false
-                }
-            }
+            Some(ext) => ext == "c" || ext == "a",
             _ => false,
         };
         if suspicious {
@@ -602,9 +565,10 @@ impl CrateInfo {
 
         // Ensure that Cargo.toml is in standard form, e.g. does not contain
         // path dependencies, so can be built standalone (see #4030).
-        let registry_toml = self.package().to_registry_toml(&self.config)?;
-        let mut actual_toml = String::new();
         let toml_path = path.join("Cargo.toml");
+        let ws = Workspace::new(&toml_path.canonicalize()?, &self.config)?;
+        let registry_toml = self.package().to_registry_toml(&ws)?;
+        let mut actual_toml = String::new();
         fs::File::open(&toml_path)?.read_to_string(&mut actual_toml)?;
 
         if !actual_toml.contains("AUTOMATICALLY GENERATED BY CARGO") {
