@@ -1,17 +1,13 @@
 use std::env::{self, VarError};
 use std::fmt::{self, Write};
 
-use failure::Error;
-use itertools::Itertools;
+use anyhow::{format_err, Error};
 use semver::Version;
 use textwrap::fill;
 
-use config::{package_field_for_feature, Config, PackageKey};
-use errors::*;
-use util::vec_opt_iter;
-
-pub const RUST_MAINT: &'static str =
-    "Debian Rust Maintainers <pkg-rust-maintainers@alioth-lists.debian.net>";
+use crate::config::{package_field_for_feature, Config, PackageKey};
+use crate::errors::*;
+use crate::util::vec_opt_iter;
 
 pub struct Source {
     name: String,
@@ -38,14 +34,14 @@ pub struct Package {
     provides: Vec<String>,
     summary: String,
     description: String,
-    boilerplate: String,
     extra_lines: Vec<String>,
 }
 
 pub struct PkgTest {
     name: String,
     crate_name: String,
-    version: Version,
+    feature: String,
+    version: String,
     extra_test_args: Vec<String>,
     depends: Vec<String>,
     extra_restricts: Vec<String>,
@@ -72,6 +68,7 @@ impl fmt::Display for Source {
         if !self.x_cargo.is_empty() {
             writeln!(f, "X-Cargo-Crate: {}", self.x_cargo)?;
         }
+        writeln!(f, "Rules-Requires-Root: no")?;
 
         Ok(())
     }
@@ -116,14 +113,20 @@ impl fmt::Display for PkgTest {
             self.version,
             self.extra_test_args.join(" ")
         )?;
+        writeln!(f, "Features: test-name={}:{}", &self.name, &self.feature)?;
+        // TODO: drop the below workaround when rust-lang/cargo#5133 is fixed.
+        // The downside of our present work-around is that more dependencies
+        // must be installed, which makes it harder to actually run the tests
+        let cargo_bug_fixed = false;
+        let default_deps = if cargo_bug_fixed { &self.name } else { "@" };
         if self.depends.is_empty() {
-            writeln!(f, "Depends: dh-cargo (>= 18), {}", &self.name)?;
+            writeln!(f, "Depends: dh-cargo (>= 18), {}", default_deps)?;
         } else {
             writeln!(
                 f,
                 "Depends: dh-cargo (>= 18), {}, {}",
                 self.depends.join(", "),
-                &self.name
+                default_deps
             )?;
         }
         if self.extra_restricts.is_empty() {
@@ -146,20 +149,20 @@ impl Source {
         upstream_name: &str,
         home: &str,
         lib: bool,
+        maintainer: String,
         uploaders: Vec<String>,
         build_deps: Vec<String>,
     ) -> Result<Source> {
         let pkgbase = match name_suffix {
-            None => format!("{}", basename),
+            None => basename.to_string(),
             Some(suf) => format!("{}{}", basename, suf),
         };
         let section = if lib {
             "rust"
         } else {
-            "FIXME-(source.section)"
+            "FIXME-IN-THE-SOURCE-SECTION"
         };
         let priority = "optional".to_string();
-        let maintainer = RUST_MAINT.to_string();
         let vcs_browser = format!(
             "https://salsa.debian.org/rust-team/debcargo-conf/tree/master/src/{}",
             pkgbase
@@ -175,24 +178,20 @@ impl Source {
             "".to_string()
         };
         Ok(Source {
-            name: format!("rust-{}", pkgbase),
+            name: dsc_name(&pkgbase),
             section: section.to_string(),
-            priority: priority,
-            maintainer: maintainer,
-            uploaders: uploaders,
-            standards: "4.2.0".to_string(),
-            build_deps: build_deps,
-            vcs_git: vcs_git,
-            vcs_browser: vcs_browser,
+            priority,
+            maintainer,
+            uploaders,
+            standards: "4.5.1".to_string(),
+            build_deps,
+            vcs_git,
+            vcs_browser,
             homepage: home.to_string(),
             x_cargo: cargo_crate,
         })
     }
 
-    pub fn srcname(&self) -> &str {
-        self.name.as_str()
-    }
-
     pub fn apply_overrides(&mut self, config: &Config) {
         if let Some(section) = config.section() {
             self.section = section.to_string();
@@ -229,9 +228,8 @@ impl Package {
         basename: &str,
         name_suffix: Option<&str>,
         version: &Version,
-        upstream_name: &str,
-        summary: Option<&str>,
-        description: Option<&str>,
+        summary: &str,
+        description: &str,
         feature: Option<&str>,
         f_deps: Vec<&str>,
         o_deps: Vec<String>,
@@ -240,7 +238,7 @@ impl Package {
         f_suggests: Vec<&str>,
     ) -> Result<Package> {
         let pkgbase = match name_suffix {
-            None => format!("{}", basename),
+            None => basename.to_string(),
             Some(suf) => format!("{}{}", basename, suf),
         };
         let deb_feature2 = &|p: &str, f: &str| {
@@ -286,55 +284,15 @@ impl Package {
         i.map(|i| provides.remove(i));
 
         let mut depends = vec!["${misc:Depends}".to_string()];
+        if feature.is_some() && !f_deps.contains(&"") {
+            // in dh-cargo we symlink /usr/share/doc/{$feature => $main} pkg
+            // so we always need this direct dependency, even if the feature
+            // only indirectly depends on the bare library via another
+            depends.push(deb_feature(""));
+        }
         depends.extend(f_deps.into_iter().map(deb_feature));
         depends.extend(o_deps);
 
-        let summary_default = format!("Rust crate \"{}\"", upstream_name);
-        let summary = summary.unwrap_or(&summary_default);
-        let short_desc = match feature {
-            Some(f) => match f_provides.len() {
-                0 => format!("{} - feature \"{}\"", summary, f),
-                _ => format!(
-                    "{} - feature \"{}\" and {} more",
-                    summary,
-                    f,
-                    f_provides.len()
-                ),
-            },
-            None => format!("{} - Rust source code", summary),
-        };
-
-        let long_desc = description.unwrap_or("");
-        let boilerplate = match feature {
-            None => format!(
-                "This package contains the source for the \
-                 Rust {} crate, packaged by debcargo for use \
-                 with cargo and dh-cargo.",
-                upstream_name
-            ),
-            Some(f) => format!(
-                "This metapackage enables feature \"{}\" for the \
-                 Rust {} crate, by pulling in any additional \
-                 dependencies needed by that feature.{}",
-                f,
-                upstream_name,
-                match f_provides.len() {
-                    0 => "".to_string(),
-                    1 => format!(
-                        "\n\nAdditionally, this package also provides the \
-                         \"{}\" feature.",
-                        f_provides[0],
-                    ),
-                    _ => format!(
-                        "\n\nAdditionally, this package also provides the \
-                         \"{}\", and \"{}\" features.",
-                        f_provides[..f_provides.len() - 1].join("\", \""),
-                        f_provides[f_provides.len() - 1],
-                    ),
-                },
-            ),
-        };
-
         Ok(Package {
             name: match feature {
                 None => deb_name(&pkgbase),
@@ -363,24 +321,18 @@ impl Package {
             // are arch:all and have the arch:any -dev packages depend on it.
             multi_arch: "same".to_string(),
             section: None,
-            depends: depends,
-            recommends: recommends,
-            suggests: suggests,
-            provides: provides,
-            summary: short_desc,
-            description: fill(&long_desc, 79),
-            boilerplate: fill(&boilerplate, 79),
+            depends,
+            recommends,
+            suggests,
+            provides,
+            summary: summary.to_string(),
+            description: description.to_string(),
             extra_lines: match (name_suffix, feature) {
                 (Some(_), None) => {
-                    let mut v = version.clone();
-                    v.increment_patch();
+                    let fullpkg = format!("{}-{}", basename, version);
                     vec![
-                        format!(
-                            "Replaces: {} (<< {}-~~)",
-                            deb_name(basename),
-                            deb_version(&v)
-                        ),
-                        format!("Breaks: {} (<< {}-~~)", deb_name(basename), deb_version(&v)),
+                        format!("Replaces: {}", deb_name(&fullpkg)),
+                        format!("Breaks: {}", deb_name(&fullpkg)),
                     ]
                 }
                 (_, _) => vec![],
@@ -391,11 +343,9 @@ impl Package {
     pub fn new_bin(
         basename: &str,
         name_suffix: Option<&str>,
-        upstream_name: &str,
         section: Option<&str>,
-        summary: Option<&str>,
-        description: Option<&str>,
-        boilerplate: &str,
+        summary: &str,
+        description: &str,
     ) -> Self {
         let (name, mut provides) = match name_suffix {
             None => (basename.to_string(), vec![]),
@@ -404,19 +354,9 @@ impl Package {
                 vec![format!("{} (= ${{binary:Version}})", basename)],
             ),
         };
-        let short_desc = match summary {
-            None => format!("Binaries built from the Rust {} crate", upstream_name),
-            Some(s) => s.to_string(),
-        };
-
-        let long_desc = match description {
-            None => "".to_string(),
-            Some(s) => s.to_string(),
-        };
-
         provides.push("${cargo:Provides}".to_string());
         Package {
-            name: name,
+            name,
             arch: "any".to_string(),
             multi_arch: "allowed".to_string(),
             section: section.map(|s| s.to_string()),
@@ -427,10 +367,9 @@ impl Package {
             ],
             recommends: vec!["${cargo:Recommends}".to_string()],
             suggests: vec!["${cargo:Suggests}".to_string()],
-            provides: provides,
-            summary: short_desc,
-            description: long_desc,
-            boilerplate: boilerplate.to_string(),
+            provides,
+            summary: summary.to_string(),
+            description: description.to_string(),
             extra_lines: vec![
                 "Built-Using: ${cargo:Built-Using}".to_string(),
                 "XB-X-Cargo-Built-Using: ${cargo:X-Cargo-Built-Using}".to_string(),
@@ -444,18 +383,7 @@ impl Package {
 
     fn write_description(&self, out: &mut fmt::Formatter) -> fmt::Result {
         writeln!(out, "Description: {}", self.summary)?;
-        let description = [&self.description, &self.boilerplate]
-            .iter()
-            .filter_map(|x| {
-                let x = x.trim();
-                if x.is_empty() {
-                    None
-                } else {
-                    Some(x)
-                }
-            })
-            .join("\n\n");
-        for line in description.trim().lines() {
+        for line in fill(&self.description.trim(), 79).lines() {
             let line = line.trim_end();
             if line.is_empty() {
                 writeln!(out, " .")?;
@@ -472,15 +400,11 @@ impl Package {
         if let Some(section) = config.package_section(key) {
             self.section = Some(section.to_string());
         }
-
-        if let Some((s, d)) = config.package_summary(key) {
-            if !s.is_empty() {
-                self.summary = s.to_string();
-            }
-
-            if !d.is_empty() {
-                self.description = d.to_string();
-            }
+        if let Some(summary) = config.package_summary(key) {
+            self.summary = summary.to_string();
+        }
+        if let Some(description) = config.package_description(key) {
+            self.description = description.to_string();
         }
 
         self.depends.extend(package_field_for_feature(
@@ -513,17 +437,19 @@ impl PkgTest {
     pub fn new(
         name: &str,
         crate_name: &str,
-        version: &Version,
+        feature: &str,
+        version: &str,
         extra_test_args: Vec<&str>,
-        depends: &Vec<String>,
+        depends: &[String],
         extra_restricts: Vec<&str>,
     ) -> Result<PkgTest> {
         Ok(PkgTest {
             name: name.to_string(),
             crate_name: crate_name.to_string(),
-            version: version.clone(),
+            feature: feature.to_string(),
+            version: version.to_string(),
             extra_test_args: extra_test_args.iter().map(|x| x.to_string()).collect(),
-            depends: depends.clone(),
+            depends: depends.to_vec(),
             extra_restricts: extra_restricts.iter().map(|x| x.to_string()).collect(),
         })
     }
@@ -539,7 +465,11 @@ pub fn deb_version(v: &Version) -> String {
     s
 }
 
-fn deb_name(name: &str) -> String {
+pub fn dsc_name(name: &str) -> String {
+    format!("rust-{}", name.replace('_', "-"))
+}
+
+pub fn deb_name(name: &str) -> String {
     format!("librust-{}-dev", name.replace('_', "-"))
 }
 
@@ -573,11 +503,11 @@ fn get_envs(keys: &[&str]) -> Result<Option<String>> {
 
 /// Determine a name and email address from environment variables.
 pub fn get_deb_author() -> Result<String> {
-    let name = get_envs(&["DEBFULLNAME", "NAME"])?.ok_or(format_err!(
-        "Unable to determine your name; please set $DEBFULLNAME or $NAME"
-    ))?;
-    let email = get_envs(&["DEBEMAIL", "EMAIL"])?.ok_or(format_err!(
-        "Unable to determine your email; please set $DEBEMAIL or $EMAIL"
-    ))?;
+    let name = get_envs(&["DEBFULLNAME", "NAME"])?.ok_or_else(|| {
+        format_err!("Unable to determine your name; please set $DEBFULLNAME or $NAME")
+    })?;
+    let email = get_envs(&["DEBEMAIL", "EMAIL"])?.ok_or_else(|| {
+        format_err!("Unable to determine your email; please set $DEBEMAIL or $EMAIL")
+    })?;
     Ok(format!("{} <{}>", name, email))
 }
