@@ -14,10 +14,9 @@ use std::fs;
 use std::io::{BufRead, BufReader, Read};
 use std::path::Path;
 
-use debian::control::RUST_MAINT;
-use errors::*;
+use crate::errors::*;
 
-const DEB_COPYRIGHT_FORMAT: &'static str =
+const DEB_COPYRIGHT_FORMAT: &str =
     "\
      https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/";
 
@@ -64,7 +63,7 @@ pub struct DebCopyright {
 
 impl fmt::Display for DebCopyright {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Format: {}\n", self.format)?;
+        writeln!(f, "Format: {}", self.format)?;
         write!(f, "{}", self.upstream)?;
 
         for file in &self.files {
@@ -92,16 +91,16 @@ impl DebCopyright {
 
 impl fmt::Display for UpstreamInfo {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Upstream-Name: {}\n", self.name)?;
+        writeln!(f, "Upstream-Name: {}", self.name)?;
         write!(f, "Upstream-Contact:")?;
         if self.contacts.len() > 1 {
-            write!(f, "\n")?;
+            writeln!(f)?;
         }
         for contact in &self.contacts {
-            write!(f, " {}\n", contact)?;
+            writeln!(f, " {}", contact)?;
         }
         if !self.source.is_empty() {
-            write!(f, "Source: {}\n", self.source)?;
+            writeln!(f, "Source: {}", self.source)?;
         }
 
         Ok(())
@@ -110,9 +109,9 @@ impl fmt::Display for UpstreamInfo {
 
 impl UpstreamInfo {
     fn new(name: String, authors: &[String], repo: &str) -> UpstreamInfo {
-        assert!(authors.len() > 0);
+        assert!(!authors.is_empty());
         UpstreamInfo {
-            name: name,
+            name,
             contacts: authors.to_vec(),
             source: repo.to_string(),
         }
@@ -121,17 +120,17 @@ impl UpstreamInfo {
 
 impl fmt::Display for Files {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Files: {}\n", self.files)?;
+        writeln!(f, "Files: {}", self.files)?;
         write!(f, "Copyright:")?;
         if self.copyright.len() > 1 {
-            write!(f, "\n")?;
+            writeln!(f)?;
         }
         for copyright in &self.copyright {
-            write!(f, " {}\n", copyright)?;
+            writeln!(f, " {}", copyright)?;
         }
-        write!(f, "License: {}\n", self.license)?;
+        writeln!(f, "License: {}", self.license)?;
         if !self.comment.is_empty() {
-            write!(f, "Comment:\n")?;
+            writeln!(f, "Comment:")?;
             format_para!(f, &self.comment)?;
         }
         Ok(())
@@ -140,7 +139,7 @@ impl fmt::Display for Files {
 
 impl Files {
     pub fn new<T: ToString>(name: &str, notice: &[T], license: &str, comment: &str) -> Files {
-        assert!(notice.len() > 0);
+        assert!(!notice.is_empty());
         Files {
             files: name.to_string(),
             copyright: notice.iter().map(|s| s.to_string()).collect(),
@@ -160,7 +159,7 @@ impl Files {
 
 impl fmt::Display for License {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "License: {}\n", self.name)?;
+        writeln!(f, "License: {}", self.name)?;
         format_para!(f, &self.text)?;
         Ok(())
     }
@@ -168,10 +167,7 @@ impl fmt::Display for License {
 
 impl License {
     fn new(name: String, text: String) -> License {
-        License {
-            name: name,
-            text: text,
-        }
+        License { name, text }
     }
 }
 
@@ -193,9 +189,8 @@ macro_rules! default_files {
 fn gen_files(debsrcdir: &Path) -> Result<Vec<Files>> {
     let mut copyright_notices = BTreeMap::new();
 
-    let copyright_notice_re = try!(regex::Regex::new(
-        r"(?:[Cc]opyright|©)(?:\s|[©:,()Cc<])*\b(\d{4}\b.*)$"
-    ));
+    let copyright_notice_re =
+        regex::Regex::new(r"(?:[Cc]opyright|©)(?:\s|[©:,()Cc<])*\b(\d{4}\b.*)$")?;
 
     // Get current working directory and move inside the extracted source of
     // crate. This is necessary so as to capture correct path for files in
@@ -208,10 +203,10 @@ fn gen_files(debsrcdir: &Path) -> Result<Vec<Files>> {
     // which again messes debian/copyright.
     // Use of . creates paths in format ./src/ which is acceptable.
     for entry in walkdir::WalkDir::new(".").sort_by(|a, b| a.file_name().cmp(b.file_name())) {
-        let entry = try!(entry);
+        let entry = entry?;
         if entry.file_type().is_file() {
             let copyright_file = entry.path().to_str().unwrap().to_string();
-            let file = try!(fs::File::open(entry.path()));
+            let file = fs::File::open(entry.path())?;
             let reader = BufReader::new(file);
             for line in reader.lines() {
                 if let Ok(line) = line {
@@ -252,7 +247,7 @@ fn gen_files(debsrcdir: &Path) -> Result<Vec<Files>> {
 
 fn get_licenses(license: &str) -> Result<Vec<License>> {
     let mut licenses = BTreeMap::new();
-    let sep = regex::Regex::new(r"(?i:(or|/|and))")?;
+    let sep = regex::Regex::new(r"(?i:(\s(or|and)\s|/))")?;
     let known_licenses = vec![
         ("agpl-3.0", include_str!("licenses/AGPL-3.0")),
         ("apache-2.0", include_str!("licenses/Apache-2.0")),
@@ -266,6 +261,7 @@ fn get_licenses(license: &str) -> Result<Vec<License>> {
         ("lgpl-2.1", include_str!("licenses/LGPL-2.1")),
         ("lgpl-3.0", include_str!("licenses/LGPL-3.0")),
         ("mit", include_str!("licenses/MIT")),
+        ("mitnfa", include_str!("licenses/MITNFA")),
         ("mpl-1.1", include_str!("licenses/MPL-1.1")),
         ("mpl-2.0", include_str!("licenses/MPL-2.0")),
         ("unlicense", include_str!("licenses/Unlicense")),
@@ -274,9 +270,14 @@ fn get_licenses(license: &str) -> Result<Vec<License>> {
     .into_iter()
     .collect::<BTreeMap<_, _>>();
 
-    let lses: Vec<&str> = sep.split(license).filter(|s| s.len() != 0).collect();
+    let lses: Vec<&str> = sep.split(license).filter(|s| !s.is_empty()).collect();
     for ls in lses {
-        let lname = ls.trim().to_lowercase().trim_end_matches('+').to_string();
+        let lname = ls
+            .trim()
+            .to_lowercase()
+            .trim_end_matches('+')
+            .trim_end_matches("-or-later")
+            .to_string();
         let text = match known_licenses.get(lname.as_str()) {
             Some(s) => s.to_string(),
             None => "FIXME (overlay): Unrecognized crate license, please find the \
@@ -337,7 +338,8 @@ pub fn debian_copyright(
     package: &package::Package,
     srcdir: &Path,
     manifest: &manifest::Manifest,
-    uploaders: &Vec<&str>,
+    maintainer: &str,
+    uploaders: &[&str],
     year_range: (i32, i32),
     guess_harder: bool,
 ) -> Result<DebCopyright> {
@@ -381,7 +383,7 @@ pub fn debian_copyright(
     } else {
         format!("{}-{}", y0, y1)
     };
-    let mut deb_notice = vec![format!("{} {}", years, RUST_MAINT)];
+    let mut deb_notice = vec![format!("{} {}", years, maintainer)];
     deb_notice.extend(uploaders.iter().map(|s| format!("{} {}", years, s)));
     files.push(Files::new("debian/*", &deb_notice, &crate_license, ""));
 
@@ -423,3 +425,6 @@ pub fn debian_copyright(
 
     Ok(DebCopyright::new(upstream, &files, &licenses))
 }
+
+#[cfg(test)]
+mod tests;
