@@ -7,8 +7,9 @@ use semver_parser::range::*;
 use std::cmp;
 use std::fmt;
 
-use config::Config;
-use errors::*;
+use crate::config::Config;
+use crate::debian;
+use crate::errors::*;
 
 #[derive(Eq, Clone)]
 enum V {
@@ -337,7 +338,7 @@ pub fn deb_dep(config: &Config, dep: &Dependency) -> Result<Vec<String>> // resu
     Ok(deps)
 }
 
-pub fn deb_deps(config: &Config, cdeps: &Vec<Dependency>) -> Result<Vec<String>> // result is a AND-clause
+pub fn deb_deps(config: &Config, cdeps: &[Dependency]) -> Result<Vec<String>> // result is a AND-clause
 {
     let mut deps = Vec::new();
     for dep in cdeps {
@@ -350,7 +351,7 @@ pub fn deb_deps(config: &Config, cdeps: &Vec<Dependency>) -> Result<Vec<String>>
 
 pub fn deb_dep_add_nocheck(x: &str) -> String {
     x.to_string()
-        .split("|")
+        .split('|')
         .map(|x| x.trim_end().to_string() + " <!nocheck> ")
         .join("|")
         .trim_end()
