@@ -12,8 +12,8 @@ fn source_package_override() {
 
     let config = config.unwrap();
 
-    assert!(config.is_source_present());
-    assert!(config.is_packages_present());
+    assert!(config.source.is_some());
+    assert!(config.packages.is_some());
 
     let policy = config.policy_version();
     assert!(policy.is_some());
@@ -32,18 +32,23 @@ fn source_package_override() {
 
     let config = config.unwrap();
 
-    assert!(config.is_source_present());
+    assert!(config.source.is_some());
 
     let section = config.section();
     assert!(section.is_some());
     assert_eq!(section.unwrap(), "rust");
 
-    assert!(config.is_packages_present());
+    assert!(config.packages.is_some());
     let sd = config.package_summary(PackageKey::Bin);
     assert!(sd.is_some());
 
-    if let Some((s, d)) = sd {
+    if let Some(s) = sd {
         assert_eq!(s, "Tool to create Debian package from Rust crate");
+    }
+
+    let sd = config.package_description(PackageKey::Bin);
+    assert!(sd.is_some());
+    if let Some(d) = sd {
         assert_eq!(
             d,
             "\
@@ -65,7 +70,7 @@ fn sd_top_level() {
 
     let config = config.unwrap();
 
-    assert!(config.is_source_present());
+    assert!(config.source.is_some());
 
     let section = config.section();
     assert!(section.is_some());
